#include <Wire.h>
#include "Adafruit_SGP30.h" 
#include <DHT.h>
#include <U8g2lib.h>

int TVOC;
int H2;
int CO2;
int EtOH;


U8G2_SSD1306_128X64_NONAME_F_SW_I2C
u8g2(U8G2_R0, /* clock=*/ 12, /* data=*/ 14, /* reset=*/ U8X8_PIN_NONE);   // All Boards without Reset of the Display

#define DHTPIN 0
#define DHTTYPE DHT11   // DHT 11
DHT dht(DHTPIN, DHTTYPE);

Adafruit_SGP30 sgp;

uint32_t getAbsoluteHumidity(float temperature, float humidity) {
  // approximation formula from Sensirion SGP30 Driver Integration chapter 3.15
  const float absoluteHumidity = 216.7f * ((humidity / 100.0f) * 6.112f * exp((17.62f * temperature) / (243.12f + temperature)) / (273.15f + temperature)); // [g/m^3]
  const uint32_t absoluteHumidityScaled = static_cast<uint32_t>(1000.0f * absoluteHumidity); // [mg/m^3]
  return absoluteHumidityScaled;
}

void setup() {
  Serial.begin(115200);
  /*while (!Serial) {
    delay(10);  // Wait for serial console to open!
  }*/
  u8g2.begin();
  
  dht.begin(); 
  Serial.println("SGP30 test");

  if (! sgp.begin()) {
    Serial.println("Sensor not found :(");
    while (1);
  }
  Serial.print("Found SGP30 serial #");
  Serial.print(sgp.serialnumber[0], HEX);
  Serial.print(sgp.serialnumber[1], HEX);
  Serial.println(sgp.serialnumber[2], HEX);
 

  // If you have a baseline measurement from before you can assign it to start, to 'self-calibrate'
  //sgp.setIAQBaseline(0x8E68, 0x8F41);  // Will vary for each sensor!
}
int counter = 0;





void loop() {
  // If you have a temperature / humidity sensor, you can set the absolute humidity to enable the humditiy compensation for the air quality signals
  //float temperature = 22.1; // [°C]
  //float humidity = 45.2; // [%RH]
  //sgp.setHumidity(getAbsoluteHumidity(temperature, humidity));
 float h = dht.readHumidity();
 float t = dht.readTemperature(); // Read temperature as Celsius (the default)
 float f = dht.readTemperature(true); // Read temperature as Fahrenheit (isFahrenheit = true)

 if (isnan(h) || isnan(t) || isnan(f)) {
   Serial.println("Failed to read from DHT sensor!");
   return;
 }
    Serial.print( "Temperature (C)= " );
    Serial.println( t);
    Serial.print( " - Temperature (F)= " );
    Serial.println(f);
    Serial.print( " - Humidity = " );
    Serial.print( h);
    Serial.println( "%" );
    Serial.println( "");



  if (! sgp.IAQmeasure()) {
    Serial.println("Measurement failed");
    return;
  }

  Serial.print("TVOC "); Serial.print(sgp.TVOC); Serial.println(" ppb\t");
  Serial.print("eCO2 "); Serial.print(sgp.eCO2); Serial.println(" ppm");

  if (! sgp.IAQmeasureRaw()) {
    Serial.println("Raw Measurement failed");
    return;
  }
  Serial.print("Raw H2 "); Serial.print(sgp.rawH2); Serial.println(" \t");
  Serial.print("Raw Ethanol "); Serial.print(sgp.rawEthanol); Serial.println("");
  Serial.println(" ");
  

  char chart[10];                
  dtostrf(t, 3, 1, chart);
  char charh[10];                
  dtostrf(h, 3, 1, charh);

  TVOC = sgp.TVOC;
  char charTVOC[10];                
  dtostrf(TVOC, 3, 0, charTVOC);

  H2 = sgp.rawH2;
  char charH2[10];                
  dtostrf(H2, 3, 0, charH2);

  CO2 = sgp.eCO2;
  char charCO2[10];                
  dtostrf(CO2, 3, 0, charCO2);
  
  EtOH = sgp.rawEthanol;
  char charEtOH[10];                
  dtostrf(EtOH, 3, 0, charEtOH);


  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_courB08_tf);
  
  u8g2.drawStr(10,10, "Weather station");
  u8g2.setCursor(10, 23);
  u8g2.print(chart);
  u8g2.print(" C"); 
  u8g2.setCursor(70, 23);
  u8g2.print(charh);
  u8g2.print(" %");
  u8g2.setCursor(10, 33);  
  u8g2.print("TVOC: ");  
  u8g2.print(charTVOC);
  u8g2.print(" ppb");  
  u8g2.setCursor(10,43);  
  u8g2.print("CO2: ");    
  u8g2.print(charCO2);  
  u8g2.print(" ppm");   
  u8g2.setCursor(10,53);  
  u8g2.print("H2: ");  
  u8g2.print(charH2);  
  u8g2.setCursor(10,63);  
  u8g2.print("EtOH: ");  
  u8g2.print(charEtOH);  

  u8g2.sendBuffer();
  
  
 
  delay(1000);

  counter++;
  if (counter == 30) {
    counter = 0;

    uint16_t TVOC_base, eCO2_base;
    if (! sgp.getIAQBaseline(&eCO2_base, &TVOC_base)) {
      Serial.println("Failed to get baseline readings");
      return;
    }
    Serial.print("****Baseline values: eCO2: 0x"); Serial.print(eCO2_base, HEX);
    Serial.print(" & TVOC: 0x"); Serial.println(TVOC_base, HEX);
  }
}
